@@ -18,10 +18,8 @@
                     class(object$finalModel)[1],
                     pamrtrained = 
                     {
-                      library(pamr)
                       if(is.null(object$finalModel$xData)) stop("the train$finalModel object needs an xData object")
-                      varImp(
-                             object$finalModel, 
+                      varImp(object$finalModel, 
                              threshold = ifelse(is.null(object$bestTune), 0, object$bestTune[,1]),
                              data = object$finalModel$xData)         
                     },
@@ -33,9 +31,8 @@
                     },
                     varImp(object$finalModel, ...))
     } else {
-      isX <- which(!(dimnames(object$trainingData)[[2]] %in% ".outcome"))
-      imp <- filterVarImp(
-                          object$trainingData[, isX],
+      isX <- which(!(colnames(object$trainingData) %in% ".outcome"))
+      imp <- filterVarImp(object$trainingData[, isX,drop = FALSE],
                           object$trainingData[, -isX],         
                           nonpara = nonpara,
                           ...)   
@@ -58,3 +55,35 @@
   
   structure(out, class = "varImp.train")
 }
+
+
+"varImp.train" <- function(object, useModel = TRUE, nonpara = TRUE, scale = TRUE, ...) {
+  code <- object$modelInfo
+  if(is.null(code$varImp)) useModel <- FALSE
+  if(useModel) {
+    checkInstall(code$library)
+    for(i in seq(along = code$library)) 
+      do.call("require", list(package = code$library[i]))
+    imp <- code$varImp(object$finalModel, ...)
+    modelName <- object$method
+  } else {
+    isX <- which(!(colnames(object$trainingData) %in% ".outcome"))
+    imp <- filterVarImp(object$trainingData[, isX,drop = FALSE],
+                        object$trainingData[, -isX],         
+                        nonpara = nonpara,
+                        ...)   
+    modelName <- ifelse(is.factor(object$trainingData[, -isX]),
+                        "ROC curve",
+                        ifelse(nonpara, "loess r-squared", "Linear model"))  
+  }
+  if(scale) {
+    if(class(object$finalModel)[1] == "pamrtrained") imp <- abs(imp)
+    imp <- imp - min(imp, na.rm = TRUE) 
+    imp <- imp/max(imp, na.rm = TRUE)*100      
+  }
+  out <- list(importance = imp,
+              model = modelName,
+              calledFrom = "varImp")
+  
+  structure(out, class = "varImp.train")
+}
